import getOppositePlacement from "../utils/getOppositePlacement.js";
import getVariationPlacement from "../utils/getVariationPlacement.js";
import getOppositeVariationPlacement from "../utils/getOppositeVariationPlacement.js";
import getBasePlacement from "../utils/getBasePlacement.js";
import mergePaddingObject from "../utils/mergePaddingObject.js";
import expandToHashMap from "../utils/expandToHashMap.js";
import { basePlacements, right, bottom, left, top, start } from "../enums.js";

function flip(_ref) {
  var state = _ref.state,
      options = _ref.options,
      name = _ref.name;
  var placement = state.placement;
  var defaultFallbackPlacements = [getOppositePlacement(state.options.placement)];
  var _options$fallbackPlac = options.fallbackPlacements,
      fallbackPlacements = _options$fallbackPlac === void 0 ? defaultFallbackPlacements : _options$fallbackPlac,
      _options$padding = options.padding,
      padding = _options$padding === void 0 ? 0 : _options$padding,
      _options$checkVariati = options.checkVariation,
      checkVariation = _options$checkVariati === void 0 ? true : _options$checkVariati;
  var overflow = state.modifiersData['detectOverflow:flip'].overflowOffsets;
  var flipIndex = state.modifiersData[name].index;
  var paddingObject = mergePaddingObject(typeof padding !== 'number' ? padding : expandToHashMap(padding, basePlacements));
  var placementOrder = [state.options.placement].concat(fallbackPlacements);
  var flippedPlacement = placementOrder[flipIndex];

  if (!flippedPlacement && placement !== state.options.placement) {
    state.placement = state.options.placement;
    state.reset = true;
    return state;
  }

  if (!flippedPlacement && placement === state.options.placement) {
    return state;
  }

  var basePlacement = getBasePlacement(flippedPlacement);
  var fitsOnMainAxis = overflow[basePlacement] + paddingObject[basePlacement] <= 0; // Check alt axis to see if we can switch variation placements

  var variation = getVariationPlacement(flippedPlacement);

  if (variation && checkVariation && fitsOnMainAxis) {
    var isVertical = [top, bottom].includes(basePlacement);
    var isStartVariation = variation === start;
    var len = isVertical ? 'width' : 'height';
    var sideA = isStartVariation ? isVertical ? right : bottom : isVertical ? left : top;
    var sideB = isStartVariation ? isVertical ? left : top : isVertical ? right : bottom;
    var fitsA = overflow[sideA] + paddingObject[sideA] <= 0;
    var fitsB = overflow[sideB] + paddingObject[sideB] <= 0;
    var oppositeVariation = getOppositeVariationPlacement(flippedPlacement);
    var storedPlacement = state.modifiersData[name + "#persistent"];
    var condition = state.measures.popper[len] < state.measures.reference[len] ? fitsA && !fitsB : !fitsA && fitsB;

    if (condition) {
      state.modifiersData[name + "#persistent"] = oppositeVariation;
      flippedPlacement = oppositeVariation;
    } else if (storedPlacement) {
      state.modifiersData[name + "#persistent"] = flippedPlacement;
      flippedPlacement = storedPlacement;
    }
  }

  if (!fitsOnMainAxis) {
    state.modifiersData[name].index += 1;
    state.reset = true;
    return state;
  } else if (fitsOnMainAxis && state.placement !== flippedPlacement) {
    state.placement = flippedPlacement;
    state.reset = true;
    return state;
  }

  return state;
}

export default {
  name: 'flip',
  enabled: true,
  phase: 'main',
  fn: flip,
  requires: ['detectOverflow:flip'],
  optionallyRequires: ['offset'],
  data: {
    index: 0
  }
};